----------------------------------------
-- SRAM
----------------------------------------

-------------------------------------------------------------------------------
-- Notes
---- 2006.10.26 
------ ƏĂ݂BSRAMւ̗v󂯎āA
------ SRAMւ̗voĤB
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

entity sram is
  
  port (
    CLK     : in    std_logic;                -- system clock
    RESET : in std_logic;               -- Reset signal
    RWIN : in std_logic;                 -- Read or Write
    DATA_IN : in std_logic_vector(31 downto 0);  -- input data
    DATA_OUT : out std_logic_vector(31 downto 0);  -- output data
    
    -- to sram
    RW      : out   std_logic;                -- Read or Write
    BW      : out   std_logic_vector(3 downto 0);
    IO_DATA : inout std_logic_vector(31 downto 0);   -- input & output data
    IO_PAR  : inout std_logic_vector(3 downto 0);    -- parity
    AADDR   : out   std_logic_vector(19 downto 0);  -- Address
    CE_1,CE_2,NCE_2 : out std_logic;          --Chip Enables
    OE : out std_logic;                 -- Output Enable
    CEN : out std_logic;                -- Clock Enable
    ADV_LD : out std_logic;             -- Barst Access
    LBO : out std_logic;                --Barst Access Order
    ZZ : out std_logic);                 -- sleep mode
    
end sram;

architecture sram_arc of sram is
  signal BUF0,BUF1,BUF2 : std_logic_vector(31 downto 0);  -- buffer
  signal RWBUF0,RWBUF1,RWBUF2 : std_logic;  -- RW buffer

begin  -- sram_arc

  -- constant
  BW <= "0000";
  CE_1 <= '1';
  CE_2 <= '0';
  NCE_2 <= '1';
  OE <= '1';
  CEN <= '1';
  ADV_LD <= '0';
  LBO <= '0';
  ZZ <= '0';  
  
  SRAM: process (CLK,RESET)
  begin  -- process SRAM

  if RESET = '1' then
    -- init
    AADDR <= (others => '0');
    BUF0 <= (others => '0');            -- o? Z?
    BUF1 <= (others => '0');
    BUF2 <= (others => '0');
    
    BW <= "0000";
    CE_1 <= '1';
    CE_2 <= '0';
    NCE_2 <= '1';
    OE <= '1';
    CEN <= '1';
    ADV_LD <= '0';
    LBO <= '0';
    ZZ <= '0';
 
  elsif CLK'event and CLK = '1' then          
    -- data send & receive
    AADDR <= ADDR(19 downto 0);
    RW <= RWIN;

    -- 3-state machine (count down)
    RWBUF2 <= RWIN;
    RWBUF1 <= RWBUF2;
    RWBUF0 <= RWBUF1;

    BUF2 <= DATA_IN when (RWIN = '0');    
    BUF1 <= BUF2;
    BUF0 <= BUF1;
    IO_DATA <= BUF0 when (RWIN = '0');
    DATA_OUT <= IO_DATA when (RWBUF0 = '1');

    -- parity ǂ[BƂ肠0000
    IOPAR <= "0000";
    
  end if;    
  end process SRAM;

end sram_arc;



