-----------------------
-- SRAM test 
-----------------------

-- Description -----------------------------------
-- USB echo test
--  READstart -(VALID)-> READdone -(BUSY)-> 
--  WRITEstart -(VALID)-> WRITEdone -(BUSY)-> back
-------------------------------------------------

---- Notes --------------------------------------------------------
-- 2007.01.18
--   Ȃ̂ŁAsram.vhdB
--   Ƃ肠A25MHzœ悤Ɋ撣B
--   ǁA50œ̂ŁÂ܂܂ŁB
-- 2006.12.14
--   SRAMɃf[^ށBރf[^̓AhXɓB
--   z50MHz̃NbNB
--
-------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

library UNISIM;
use UNISIM.VCOMPONENTS.all;

entity SRAMW is
     port(
        CLKIN : in std_logic;
        LED : out std_logic_vector(7 downto 0);

        -- to SRAM
        SRAM_RW : out std_logic; -- read or write
        SRAM_IO : inout std_logic_vector(31 downto 0);  -- IO data
        SRAM_IOP : inout std_logic_vector(3 downto 0);  -- IO parity
        SRAM_AA : out std_logic_vector(19 downto 0);    -- address
        -- to SRAM(constant)
        SRAM_CLKM0,SRAM_CLKM1 : out std_logic;                      -- SRAM clock
        SRAM_BW : out std_logic_vector(3 downto 0);
        SRAM_CEL1X,SRAM_CEH1X,SRAM_CE2,SRAM_CE2X : out std_logic;
        SRAM_OEX,SRAM_CENX,SRAM_ADVLD : out std_logic;
        SRAM_LBOX : out std_logic;
        SRAM_ZZ : out std_logic);
end SRAMW;

architecture RTL of SRAMW is
    component ROC
        port(
            O : out std_logic);
    end component;

    component IBUFG
        port(
            I : in std_logic;
            O : out std_logic);
    end component;

    component SRAM
        port(
        CLK : in std_logic;                         -- clock 50MHz
        RESET : in std_logic;                       -- reset

        MODE : in std_logic;                       -- read/write mode
        ADDR : in std_logic_vector(19 downto 0);    -- read/write address
        AOUT : out std_logic_vector(19 downto 0);   -- current data address
        RDATA : out std_logic_vector(31 downto 0);  -- read data
        RP : out std_logic_vector(3 downto 0);      -- read parity
        WDATA : in std_logic_vector(31 downto 0);   -- write data
        WP : in std_logic_vector(3 downto 0);       -- write parity
        -- to SRAM
        RW : out std_logic; -- read or write
        IO : inout std_logic_vector(31 downto 0);  -- IO data
        IOP : inout std_logic_vector(3 downto 0);  -- IO parity
        AA : out std_logic_vector(19 downto 0);    -- address
        -- to SRAM(constant)
        CLKM0,CLKM1 : out std_logic;                      -- SRAM clock
        BW : out std_logic_vector(3 downto 0);
        CEL1X,CEH1X,CE2,CE2X : out std_logic;
        OEX,CENX,ADVLD : out std_logic;
        LBOX : out std_logic;
        ZZ : out std_logic);
	end component;

    signal CLK : std_logic;
    signal RESET : std_logic;

    signal SMODE : std_logic;
    signal SRDATA,SWDATA : std_logic_vector(31 downto 0);
    signal SADDR,SAOUT : std_logic_vector(19 downto 0);
    signal SRP,SWP : std_logic_vector(3 downto 0);

    signal STATE : std_logic_vector(1 downto 0);
    signal COUNT : std_logic_vector(31 downto 0);
    signal SUBCOUNT : std_logic_vector(2 downto 0);

begin

    LED   <= STATE & COUNT(31 downto 26);   -- for debug

    RC : ROC port map (
        O => RESET);

    -- Inner clk is "CLK"
    IBG : IBUFG port map (
        I => CLKIN,
        O => CLK);

    SRAM_INST : SRAM port map (
        CLK   => CLK,
        RESET => RESET,
        MODE  => SMODE,
        ADDR  => SADDR,
        AOUT  => SAOUT,
        RDATA => SRDATA,
        RP    => SRP,
        WDATA => SWDATA,
        WP    => SWP,
        RW    => SRAM_RW,
        IO    => SRAM_IO,
        IOP   => SRAM_IOP,
        AA    => SRAM_AA,
        CLKM0  => SRAM_CLKM0,
        CLKM1  => SRAM_CLKM1,
        BW    => SRAM_BW,
        CEL1X  => SRAM_CEL1X,
        CEH1X  => SRAM_CEH1X,
        CE2   => SRAM_CE2,
        CE2X  => SRAM_CE2X,
        OEX   => SRAM_OEX,
        CENX  => SRAM_CENX,
        ADVLD => SRAM_ADVLD,
        LBOX  => SRAM_LBOX,
        ZZ    => SRAM_ZZ);

    process(CLK,RESET)
    begin
        if RESET = '1' then
            SADDR <= (others => '0');
            COUNT <= (others => '0');
		  SUBCOUNT <= (others => '0');
            STATE <= "10";
        elsif CLK'event and CLK = '1' then
			
            case STATE is
		  	 when "10" =>
		  	     SUBCOUNT <= SUBCOUNT + 1;
			 	if SUBCOUNT = "111" then
					STATE <= "00";
				else
					STATE <= "10";
				end if; 
                when "00" =>
			 	COUNT <= COUNT + 1;
			    SMODE <= '0';                       -- always write
			    SWP   <= "0000";                    -- write parity
			    SADDR <= COUNT(19 downto 0);        -- write address 
			    SWDATA <= COUNT(31 downto 0);                    -- write data
                 if COUNT(19 downto 0) = "11111111111111111111" then
                        STATE <= "01";
                    else
                        STATE <= "00";
                  end if;			 
                when "01" =>
			 	SMODE <= '1';
                    STATE <= "01";
                when others =>
                    STATE <= "01";
                end case;
        end if;
    end process;

end RTL;

